﻿using System;
using Ozeki.Media;
using Ozeki.VoIP;

namespace ASPVoipExample
{
    public class Global : System.Web.HttpApplication
    {
        private const bool CustomerCall = true;
        private static ISoftPhone softPhone;
        private static IPhoneLine phoneLine;
        private static RegState phoneLineState;
        private static IPhoneCall calloperator;
        private static IPhoneCall callcustumer;

        private static PhoneCallAudioSender operatorSender = new PhoneCallAudioSender();
        private static PhoneCallAudioReceiver operatorReceiver = new PhoneCallAudioReceiver();
        private static PhoneCallAudioSender customerSender = new PhoneCallAudioSender();
        private static PhoneCallAudioReceiver customerReceiver = new PhoneCallAudioReceiver();
        private static MediaConnector mediaConnector = new MediaConnector();

        public string operatorNumber,customerNumber;

        #region Helper Functions
        /// <summary>
        ///It initializes a softphone object  with a SIP BPX, and it  is for requisiting a SIP account that is nedded  for a SIP PBX service. It registers this SIP
        ///account to the SIP PBX through an ’IphoneLine’ object  which represents the telephoneline. 
        ///If the telephone registration is successful we have a call ready softphone. In this example the softphone can be reached by dialing the number 891.
        /// </summary>
        private void InitializeSoftPhone()
        {
            try
            {
                softPhone = SoftPhoneFactory.CreateSoftPhone(SoftPhoneFactory.GetLocalIP(), 5700, 5750);
                softPhone.IncomingCall += softPhone_IncomingCall;
                phoneLine = softPhone.CreatePhoneLine(new SIPAccount(true, "oz875", "oz875", "oz875", "oz875", "192.168.115.103", 5060));
                phoneLine.RegistrationStateChanged += phoneLine_PhoneLineInformation;

                softPhone.RegisterPhoneLine(phoneLine);

                mediaConnector.Connect(operatorReceiver, customerSender);
                mediaConnector.Connect(customerReceiver, operatorSender);
            }
            catch (Exception ex)
            {
                //MessageBox.Show(String.Format("You didn't give your local IP adress, so the program won't run properly.\n {0}", ex.Message), string.Empty, MessageBoxButtons.OK,
                //MessageBoxIcon.Error);
            }
        }

        /// <summary>
        ///  It signs up to the necessary events of a call transact.
        /// </summary>
        private void WireUpCallEvents(IPhoneCall call)
        {
            call.CallStateChanged += (call_CallStateChanged);
            call.DtmfReceived += (call_DtmfReceived);
        }

        private int counter = 0;
        /// <summary>
        /// It starts a call to an operator or an customer.
        /// </summary>
        /// <param name="IsCustomerCall">direction of the call</param>
        public void BeginCall(bool IsCustomerCall)
        {
          
            if (IsCustomerCall)
            {
                if (phoneLineState != RegState.RegistrationSucceeded)
                {
                    return;
                }
                
                callcustumer = softPhone.CreateCallObject(phoneLine,customerNumber);
                WireUpCallEvents(callcustumer);
                customerReceiver.AttachToCall(callcustumer);
                customerSender.AttachToCall(callcustumer);
                callcustumer.Start();
            }
            else
            {

                if (phoneLineState != RegState.RegistrationSucceeded)
                {
                    return;
                }
                calloperator = softPhone.CreateCallObject(phoneLine, operatorNumber);
                WireUpCallEvents(calloperator);
                operatorSender.AttachToCall(calloperator);
                operatorReceiver.AttachToCall(calloperator);
                calloperator.Start();
            }
        }

        /// <summary>
        /// It stops a call.
        /// </summary>
        /// <param name="call"></param>
        private void HangUpCall(IPhoneCall call)
        {
            if (call == calloperator)
            {
                if (callcustumer != null)
                {
                    callcustumer.HangUp();
                }
                operatorSender.Detach();
                operatorReceiver.Detach();
            }
            else
            {
                if (calloperator != null)
                {
                    calloperator.HangUp();
                }
                customerSender.Detach();
                customerReceiver.Detach();
            }
            //InvokeGUIThread(() => { ControlsEnable(); });
        }

        /// <summary>
        /// The controls of the Windows Form applications can only be modified on the GUI thread. This method grants access to the GUI thread.
        /// </summary>
        /// <param name="action"></param>
        private void InvokeGUIThread(Action action)
        {
            //  Invoke(action);
        }

        #endregion

        #region Ozeki VoIP-SIP SDK's events

        /// <summary>
        /// Occurs when phone line state has changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void phoneLine_PhoneLineInformation(object sender, RegistrationStateChangedArgs e)
        {
            phoneLineState = e.State;
        }

        /// <summary>
        /// Occurs when an incoming call request has received.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void softPhone_IncomingCall(object sender, VoIPEventArgs<IPhoneCall> e)
        {
            e.Item.Reject();
        }

        /// <summary>
        /// Occurs when the phone call state has changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void call_CallStateChanged(object sender, CallStateChangedArgs e)
        {
            if (e.State == CallState.Answered)
            {
                var call = sender as IPhoneCall;
                if (call.CallID == calloperator.CallID)
                {
                    BeginCall(CustomerCall);
                }

                return;
            }

            if (e.State.IsCallEnded())
            {
                HangUpCall((IPhoneCall)sender);
                calloperator = null;
                callcustumer = null;
            }
        }


        /// <summary>
        /// Displays DTMF signals
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void call_DtmfReceived(object sender, VoIPEventArgs<DtmfInfo> e)
        {

        }

        #endregion


        protected void Application_Start(object sender, EventArgs e)
        {
            
        }

        protected void Session_Start(object sender, EventArgs e)
        {
            InitializeSoftPhone();
        }

        protected void Application_BeginRequest(object sender, EventArgs e)
        {

        }

        protected void Application_AuthenticateRequest(object sender, EventArgs e)
        {

        }

        protected void Application_Error(object sender, EventArgs e)
        {

        }

        protected void Session_End(object sender, EventArgs e)
        {

        }

        protected void Application_End(object sender, EventArgs e)
        {

        }
    }
}